//*******************************************************
//
//	Raspberry Pi Power Lock
//
//	ATtiny13
//		hfuse:0xFB	BOD 2.7V
//		lfuse:22h	0.6MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.7.1
//				binutils	2.20.1
//
//	2012.8.22	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/eeprom.h>

#include	"main.h"

volatile uchar	WTimer;
volatile uchar	SwCountDS;
volatile uchar	SwCountSec;

//*******************************************************
//	タイマー０比較一致割り込み
//*******************************************************
ISR(TIM0_COMPA_vect){

	if (WTimer)	--WTimer;

	// スイッチカウント
	if (!(PINB & (1<<SWITCH))){
		if (++SwCountDS >= 100){
			SwCountDS = 0;
			if (SwCountSec < 255){
				SwCountSec++;
			}
		}
	} else {
		SwCountDS = 0;
		SwCountSec = 0;
	}
}


//*******************************************************
//	10ms単位時間待ち
//*******************************************************
void wait_timer(uchar tm){

	WTimer = tm;

	while(WTimer){
		WDR;
		SLEEP;
	}
}


//*******************************************************
//	電源OFF
//*******************************************************
void shutdown(void){

	PORTB &= ~(1<<PLOCK);
	PORTB &= ~(1<<LED);
	for (;;){
		WDR;
		SLEEP;
	}
}


//*******************************************************
//	メインルーチン
//*******************************************************
int main (void) {
	uchar	last_ctrl = 1;

	cli();	// 割り込み禁止

	PORTB = (1<<SWITCH)|(1<<CTRL_IN);
	DDRB = (1<<PLOCK)|(1<<LED);

	// スリープ動作設定
	MCUCR = (1<<SE);	// スリープ許可, アイドルモード

	// タイマ0設定
	TCCR0A = (1<<WGM01);			// CTCモード
	TCCR0B = (1<<CS01)|(1<<CS00);	// プリスケーラ 1/64
	OCR0A = F_CPU/64/100 - 1;		// 100Hz
	TIMSK0 = (1<<OCIE0A);	// 比較A割り込み許可

	ACSR |= (1<<ACD);	// アナログコンパレータ禁止

	sei();

	wait_timer(10);		// 100ms

	PORTB |= (1<<PLOCK);
	PORTB |= (1<<LED);

	// スイッチが放されるまで待つ
	SLEEP;
	while(!(PINB & (1<<SWITCH))){
		WDR;
		SLEEP;
	}

	// メインループ
	for (;;){
		uchar	ctrl;

		WDR;
		SLEEP;

		// スイッチ 10秒以上長押し
		if (SwCountSec >= 10){
			shutdown();	// 強制電源断
		}

		ctrl = PINB & (1<<CTRL_IN);
		if (!last_ctrl && ctrl){	// CTRL_IN 立ち上がり
			uchar	i;
			// 停止モード
			for (i = 0; i < 40; i++){
				wait_timer(50);		// 0.5秒
				PINB |= (1<<LED);	// LED点滅
			}
			shutdown();	// 電源断
		}
		last_ctrl = ctrl;
	}
}
